import 'package:fling_pickle/fling_pickle.dart';
import 'package:test/test.dart';

class Tester implements Pickleable {
  final String name;

  Tester({final this.name = 'Fred'});

  static Tester fromPickle(final Pickle pickle) =>
      Tester(name: pickle.readString('name'));

  @override
  Pickle asPickle() {
    return PickleBuilder().withString('name', name).build();
  }

  @override
  bool operator ==(final dynamic other) =>
      other is Tester && other.name == name;
}

void main() {
  group('withBool', () {
    test('assigned and read', () {
      // given
      final pickle = PickleBuilder().withBool('key', true).build();

      // when
      final result = pickle.readBool('key');

      // then
      expect(result, true);
    });
    test('without assignment', () {
      // given
      final pickle = PickleBuilder().build();

      // then
      expect(() => pickle.readBool('key'), throwsArgumentError);
    });
    test('assigned as different type', () {
      // given
      final pickle = PickleBuilder().withString('key', 'true').build();

      // then
      expect(() => pickle.readBool('key'), throwsA(isA<PickleTypeError>()));
    });
  });

  group('withDouble', () {
    test('assigned and read', () {
      // given
      final pickle = PickleBuilder().withDouble('key', 2.4).build();

      // when
      final result = pickle.readDouble('key');

      // then
      expect(result, 2.4);
    });
    test('without assignment', () {
      // given
      final pickle = PickleBuilder().build();

      // then
      expect(() => pickle.readDouble('key'), throwsArgumentError);
    });
    test('assigned as different type', () {
      // given
      final pickle = PickleBuilder().withString('key', '2.4').build();

      // then
      expect(() => pickle.readDouble('key'), throwsA(isA<PickleTypeError>()));
    });
  });

  group('withInt', () {
    test('assigned and read', () {
      // given
      final pickle = PickleBuilder().withInt('key', 2).build();

      // when
      final result = pickle.readInt('key');

      // then
      expect(result, 2);
    });
    test('without assignment', () {
      // given
      final pickle = PickleBuilder().build();

      // then
      expect(() => pickle.readInt('key'), throwsArgumentError);
    });
    test('assigned as different type', () {
      // given
      final pickle = PickleBuilder().withString('key', '2').build();

      // then
      expect(() => pickle.readInt('key'), throwsA(isA<PickleTypeError>()));
    });
  });

  group('withString', () {
    test('assigned and read', () {
      // given
      final pickle = PickleBuilder().withString('key', 'SUCCESS').build();

      // when
      final result = pickle.readString('key');

      // then
      expect(result, 'SUCCESS');
    });
    test('without assignment', () {
      // given
      final pickle = PickleBuilder().build();

      // then
      expect(() => pickle.readString('key'), throwsArgumentError);
    });
    test('assigned as different type', () {
      // given
      final pickle = PickleBuilder().withInt('key', 2).build();

      // then
      expect(() => pickle.readString('key'), throwsA(isA<PickleTypeError>()));
    });
  });

  group('withPickle', () {
    test('assigned and read', () {
      // given
      final pickle = PickleBuilder()
          .withPickle('key', PickleBuilder().withInt('lock', 2).build())
          .build();

      // when
      final result = pickle.readPickle('key');

      // then
      expect(result.readInt('lock'), 2);
    });
    test('without assignment', () {
      // given
      final pickle = PickleBuilder().build();

      // then
      expect(() => pickle.readPickle('key'), throwsArgumentError);
    });
    test('assigned as different type', () {
      // given
      final pickle = PickleBuilder().withString('key', 'Pickle').build();

      // then
      expect(() => pickle.readPickle('key'), throwsA(isA<PickleTypeError>()));
    });
  });

  group('withPickles', () {
    test('assigned and read', () {
      // given
      final pickle =
          PickleBuilder().withPickles('key', [PickleBuilder().build()]).build();

      // when
      final result = pickle.readPickles('key');

      // then
      expect(result.length, 1);
    });
    test('without assignment', () {
      // given
      final pickle = PickleBuilder().build();

      // then
      expect(() => pickle.readPickles('key'), throwsArgumentError);
    });
    test('assigned as different type', () {
      // given
      final pickle = PickleBuilder().withString('key', '[]').build();

      // then
      expect(() => pickle.readPickles('key'), throwsA(isA<PickleTypeError>()));
    });
  });

  group('withPickleable', () {
    test('assigned and read', () {
      // given
      final pickle =
          PickleBuilder().withPickleable('key', Tester(name: 'kiwi')).build();

      // when
      final result = pickle.readPickleable('key', Tester.fromPickle);

      // then
      expect(result.name, 'kiwi');
    });
    test('without assignment', () {
      // given
      final pickle = PickleBuilder().build();

      // then
      expect(() => pickle.readPickleable('key', Tester.fromPickle),
          throwsArgumentError);
    });
    test('assigned as different type', () {
      // given
      final pickle = PickleBuilder().withString('key', 'Tester').build();

      // then
      expect(() => pickle.readPickleable('key', Tester.fromPickle),
          throwsA(isA<PickleTypeError>()));
    });
  });

  group('withPickleables', () {
    test('assigned and read', () {
      // given
      final pickle = PickleBuilder().withPickleables(
          'key', [Tester(name: 'Henry'), Tester(name: 'Henrietta')]).build();

      // when
      final result = pickle.readPickleables('key', Tester.fromPickle);

      // then
      expect(result, [Tester(name: 'Henry'), Tester(name: 'Henrietta')]);
    });
    test('without assignment', () {
      // given
      final pickle = PickleBuilder().build();

      // then
      expect(() => pickle.readPickleables('key', Tester.fromPickle),
          throwsArgumentError);
    });
    test('assigned as different type', () {
      // given
      final pickle = PickleBuilder().withString('key', '2').build();

      // then
      expect(() => pickle.readPickleables('key', Tester.fromPickle),
          throwsA(isA<PickleTypeError>()));
    });
  });
}
